Module: Amp::Help

Defined in:
lib/amp/help/help.rb,
lib/amp.rb

Overview

The module covering all the help subsystems for the Amp binary.

Defined Under Namespace

Modules: HelpRegistry, HelpUI Classes: CommandHelpEntry, ErbHelpEntry, HelpEntry, MarkdownHelpEntry

Class Method Summary collapse

Class Method Details

.load_default_entriesObject

A method that loads in the default entries for the help system. Normally, I’d just put this code in the module itself, or perhaps at the end of the file, but I’m experimenting with an approach where I try to minimize the bare code, leaving only the invocation of this method to sit in the module.



279
280
281
282
283
# File 'lib/amp/help/help.rb', line 279

def self.load_default_entries
  Dir[File.join(File.dirname(__FILE__), "entries", "**")].each do |file|
    HelpEntry.from_file(file)
  end
end